import json
from .api import ApiError


class GqlClient:
    def __init__(
        self,
        http_api,
        path,
    ):
        self._http_api = http_api
        self._path = path

    def post(self, query: str, variables={}, retry=None) -> dict:
        request_payload = {"query": query}
        if variables:
            request_payload["variables"] = variables
        response = self._http_api.post(
            self._path,
            json.dumps(request_payload),
            {"Content-type": "application/json"},
            retry=retry,
        )
        payload = response.json()
        if "errors" in payload:
            message = ",".join([error["message"] for error in payload["errors"]])
            raise ApiError(message)
        elif "data" in payload:
            return payload["data"]
        else:
            raise ApiError("Malformed GraphQL response: no 'data' or 'errors' key")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: delta_enclave_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x64\x65lta_enclave_api.proto\x12\x11\x64\x65lta_enclave_api\"\x91\x01\n\x07Request\x12:\n\x0c\x64\x65ltaRequest\x18\x01 \x01(\x0b\x32\".delta_enclave_api.DataNoncePubkeyH\x00\x12?\n\x10\x65xtensionMessage\x18\x02 \x01(\x0b\x32#.delta_enclave_api.ExtensionMessageH\x00\x42\t\n\x07request\"1\n\x10\x45xtensionMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"T\n\x08Response\x12\x1c\n\x12successfulResponse\x18\x01 \x01(\x0cH\x00\x12\x1e\n\x14unsuccessfulResponse\x18\x02 \x01(\tH\x00\x42\n\n\x08response\">\n\x0f\x44\x61taNoncePubkey\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\r\n\x05nonce\x18\x02 \x01(\x0c\x12\x0e\n\x06pubkey\x18\x03 \x01(\x0c\"(\n\tDataNonce\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\r\n\x05nonce\x18\x02 \x01(\x0c\"j\n\x16SealedEncryptedMessage\x12\x36\n\x10\x65ncryptedMessage\x18\x01 \x01(\x0b\x32\x1c.delta_enclave_api.DataNonce\x12\x18\n\x10sealingKeyParams\x18\x02 \x01(\x0c\"W\n\x10\x45ncryptionHeader\x12/\n\x08\x63hilyKey\x18\x01 \x01(\x0b\x32\x1b.delta_enclave_api.ChilyKeyH\x00\x42\x12\n\x10\x65ncryptionHeader\"#\n\x08\x43hilyKey\x12\x17\n\x0f\x65ncryptionNonce\x18\x02 \x01(\x0c\" \n\rVersionHeader\x12\x0f\n\x07version\x18\x01 \x01(\x03\"\x83\x01\n\x0b\x43hunkHeader\x12\x14\n\x0c\x65xtraEntropy\x18\x01 \x01(\x0c\x12!\n\x14untrustedContentSize\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\"\n\x1auntrustedChunkContentSizes\x18\x03 \x03(\x04\x42\x17\n\x15_untrustedContentSizeb\x06proto3')



_REQUEST = DESCRIPTOR.message_types_by_name['Request']
_EXTENSIONMESSAGE = DESCRIPTOR.message_types_by_name['ExtensionMessage']
_RESPONSE = DESCRIPTOR.message_types_by_name['Response']
_DATANONCEPUBKEY = DESCRIPTOR.message_types_by_name['DataNoncePubkey']
_DATANONCE = DESCRIPTOR.message_types_by_name['DataNonce']
_SEALEDENCRYPTEDMESSAGE = DESCRIPTOR.message_types_by_name['SealedEncryptedMessage']
_ENCRYPTIONHEADER = DESCRIPTOR.message_types_by_name['EncryptionHeader']
_CHILYKEY = DESCRIPTOR.message_types_by_name['ChilyKey']
_VERSIONHEADER = DESCRIPTOR.message_types_by_name['VersionHeader']
_CHUNKHEADER = DESCRIPTOR.message_types_by_name['ChunkHeader']
Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.Request)
  })
_sym_db.RegisterMessage(Request)

ExtensionMessage = _reflection.GeneratedProtocolMessageType('ExtensionMessage', (_message.Message,), {
  'DESCRIPTOR' : _EXTENSIONMESSAGE,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.ExtensionMessage)
  })
_sym_db.RegisterMessage(ExtensionMessage)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.Response)
  })
_sym_db.RegisterMessage(Response)

DataNoncePubkey = _reflection.GeneratedProtocolMessageType('DataNoncePubkey', (_message.Message,), {
  'DESCRIPTOR' : _DATANONCEPUBKEY,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.DataNoncePubkey)
  })
_sym_db.RegisterMessage(DataNoncePubkey)

DataNonce = _reflection.GeneratedProtocolMessageType('DataNonce', (_message.Message,), {
  'DESCRIPTOR' : _DATANONCE,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.DataNonce)
  })
_sym_db.RegisterMessage(DataNonce)

SealedEncryptedMessage = _reflection.GeneratedProtocolMessageType('SealedEncryptedMessage', (_message.Message,), {
  'DESCRIPTOR' : _SEALEDENCRYPTEDMESSAGE,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.SealedEncryptedMessage)
  })
_sym_db.RegisterMessage(SealedEncryptedMessage)

EncryptionHeader = _reflection.GeneratedProtocolMessageType('EncryptionHeader', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTIONHEADER,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.EncryptionHeader)
  })
_sym_db.RegisterMessage(EncryptionHeader)

ChilyKey = _reflection.GeneratedProtocolMessageType('ChilyKey', (_message.Message,), {
  'DESCRIPTOR' : _CHILYKEY,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.ChilyKey)
  })
_sym_db.RegisterMessage(ChilyKey)

VersionHeader = _reflection.GeneratedProtocolMessageType('VersionHeader', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONHEADER,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.VersionHeader)
  })
_sym_db.RegisterMessage(VersionHeader)

ChunkHeader = _reflection.GeneratedProtocolMessageType('ChunkHeader', (_message.Message,), {
  'DESCRIPTOR' : _CHUNKHEADER,
  '__module__' : 'delta_enclave_api_pb2'
  # @@protoc_insertion_point(class_scope:delta_enclave_api.ChunkHeader)
  })
_sym_db.RegisterMessage(ChunkHeader)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REQUEST._serialized_start=47
  _REQUEST._serialized_end=192
  _EXTENSIONMESSAGE._serialized_start=194
  _EXTENSIONMESSAGE._serialized_end=243
  _RESPONSE._serialized_start=245
  _RESPONSE._serialized_end=329
  _DATANONCEPUBKEY._serialized_start=331
  _DATANONCEPUBKEY._serialized_end=393
  _DATANONCE._serialized_start=395
  _DATANONCE._serialized_end=435
  _SEALEDENCRYPTEDMESSAGE._serialized_start=437
  _SEALEDENCRYPTEDMESSAGE._serialized_end=543
  _ENCRYPTIONHEADER._serialized_start=545
  _ENCRYPTIONHEADER._serialized_end=632
  _CHILYKEY._serialized_start=634
  _CHILYKEY._serialized_end=669
  _VERSIONHEADER._serialized_start=671
  _VERSIONHEADER._serialized_end=703
  _CHUNKHEADER._serialized_start=706
  _CHUNKHEADER._serialized_end=837
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meta_sink.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fmeta_sink.proto\x12\tmeta_sink\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"C\n\x07ZipFile\x12+\n\nsingleFile\x18\x01 \x01(\x0b\x32\x15.meta_sink.SingleFileH\x00\x42\x0b\n\tselection\"{\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12!\n\x03raw\x18\x03 \x01(\x0b\x32\x12.meta_sink.RawFileH\x00\x12!\n\x03zip\x18\x04 \x01(\x0b\x32\x12.meta_sink.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xb4\x01\n\x1bMetaSinkWorkerConfiguration\x12#\n\x05input\x18\x01 \x01(\x0b\x32\x14.meta_sink.SinkInput\x12\x1d\n\x15\x61\x63\x63\x65ssTokenDependency\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x64\x41\x63\x63ountId\x18\x03 \x01(\t\x12\x14\n\x0c\x61udienceName\x18\x04 \x01(\t\x12\x17\n\napiVersion\x18\x05 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_apiVersionb\x06proto3')



_SINGLEFILE = DESCRIPTOR.message_types_by_name['SingleFile']
_RAWFILE = DESCRIPTOR.message_types_by_name['RawFile']
_ZIPFILE = DESCRIPTOR.message_types_by_name['ZipFile']
_SINKINPUT = DESCRIPTOR.message_types_by_name['SinkInput']
_METASINKWORKERCONFIGURATION = DESCRIPTOR.message_types_by_name['MetaSinkWorkerConfiguration']
SingleFile = _reflection.GeneratedProtocolMessageType('SingleFile', (_message.Message,), {
  'DESCRIPTOR' : _SINGLEFILE,
  '__module__' : 'meta_sink_pb2'
  # @@protoc_insertion_point(class_scope:meta_sink.SingleFile)
  })
_sym_db.RegisterMessage(SingleFile)

RawFile = _reflection.GeneratedProtocolMessageType('RawFile', (_message.Message,), {
  'DESCRIPTOR' : _RAWFILE,
  '__module__' : 'meta_sink_pb2'
  # @@protoc_insertion_point(class_scope:meta_sink.RawFile)
  })
_sym_db.RegisterMessage(RawFile)

ZipFile = _reflection.GeneratedProtocolMessageType('ZipFile', (_message.Message,), {
  'DESCRIPTOR' : _ZIPFILE,
  '__module__' : 'meta_sink_pb2'
  # @@protoc_insertion_point(class_scope:meta_sink.ZipFile)
  })
_sym_db.RegisterMessage(ZipFile)

SinkInput = _reflection.GeneratedProtocolMessageType('SinkInput', (_message.Message,), {
  'DESCRIPTOR' : _SINKINPUT,
  '__module__' : 'meta_sink_pb2'
  # @@protoc_insertion_point(class_scope:meta_sink.SinkInput)
  })
_sym_db.RegisterMessage(SinkInput)

MetaSinkWorkerConfiguration = _reflection.GeneratedProtocolMessageType('MetaSinkWorkerConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _METASINKWORKERCONFIGURATION,
  '__module__' : 'meta_sink_pb2'
  # @@protoc_insertion_point(class_scope:meta_sink.MetaSinkWorkerConfiguration)
  })
_sym_db.RegisterMessage(MetaSinkWorkerConfiguration)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SINGLEFILE._serialized_start=30
  _SINGLEFILE._serialized_end=56
  _RAWFILE._serialized_start=58
  _RAWFILE._serialized_end=67
  _ZIPFILE._serialized_start=69
  _ZIPFILE._serialized_end=136
  _SINKINPUT._serialized_start=138
  _SINKINPUT._serialized_end=261
  _METASINKWORKERCONFIGURATION._serialized_start=264
  _METASINKWORKERCONFIGURATION._serialized_end=444
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import data_room_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import identity_endorsement_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataRoomStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataRoomStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataRoomStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Active: _DataRoomStatus.ValueType  # 0
    Stopped: _DataRoomStatus.ValueType  # 1

class DataRoomStatus(_DataRoomStatus, metaclass=_DataRoomStatusEnumTypeWrapper): ...

Active: DataRoomStatus.ValueType  # 0
Stopped: DataRoomStatus.ValueType  # 1
global___DataRoomStatus = DataRoomStatus

@typing_extensions.final
class GcgRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USERAUTH_FIELD_NUMBER: builtins.int
    CREATEDATAROOMREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVECURRENTDATAROOMCONFIGURATIONREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSREQUEST_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGREQUEST_FIELD_NUMBER: builtins.int
    PUBLISHDATASETTODATAROOMREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSREQUEST_FIELD_NUMBER: builtins.int
    REMOVEPUBLISHEDDATASETREQUEST_FIELD_NUMBER: builtins.int
    EXECUTECOMPUTEREQUEST_FIELD_NUMBER: builtins.int
    JOBSTATUSREQUEST_FIELD_NUMBER: builtins.int
    GETRESULTSREQUEST_FIELD_NUMBER: builtins.int
    CREATECONFIGURATIONCOMMITREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVECONFIGURATIONCOMMITREQUEST_FIELD_NUMBER: builtins.int
    EXECUTEDEVELOPMENTCOMPUTEREQUEST_FIELD_NUMBER: builtins.int
    GENERATEMERGEAPPROVALSIGNATUREREQUEST_FIELD_NUMBER: builtins.int
    MERGECONFIGURATIONCOMMITREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVECONFIGURATIONCOMMITAPPROVERSREQUEST_FIELD_NUMBER: builtins.int
    CASAUXILIARYSTATEREQUEST_FIELD_NUMBER: builtins.int
    READAUXILIARYSTATEREQUEST_FIELD_NUMBER: builtins.int
    RETRIEVEUSEDAIRLOCKQUOTAREQUEST_FIELD_NUMBER: builtins.int
    GETRESULTSSIZEREQUEST_FIELD_NUMBER: builtins.int
    ENDORSEMENTREQUEST_FIELD_NUMBER: builtins.int
    @property
    def userAuth(self) -> global___UserAuth: ...
    @property
    def createDataRoomRequest(self) -> global___CreateDataRoomRequest: ...
    @property
    def retrieveDataRoomRequest(self) -> global___RetrieveDataRoomRequest: ...
    @property
    def retrieveCurrentDataRoomConfigurationRequest(self) -> global___RetrieveCurrentDataRoomConfigurationRequest: ...
    @property
    def retrieveDataRoomStatusRequest(self) -> global___RetrieveDataRoomStatusRequest: ...
    @property
    def updateDataRoomStatusRequest(self) -> global___UpdateDataRoomStatusRequest: ...
    @property
    def retrieveAuditLogRequest(self) -> global___RetrieveAuditLogRequest: ...
    @property
    def publishDatasetToDataRoomRequest(self) -> global___PublishDatasetToDataRoomRequest: ...
    @property
    def retrievePublishedDatasetsRequest(self) -> global___RetrievePublishedDatasetsRequest: ...
    @property
    def removePublishedDatasetRequest(self) -> global___RemovePublishedDatasetRequest: ...
    @property
    def executeComputeRequest(self) -> global___ExecuteComputeRequest: ...
    @property
    def jobStatusRequest(self) -> global___JobStatusRequest: ...
    @property
    def getResultsRequest(self) -> global___GetResultsRequest: ...
    @property
    def createConfigurationCommitRequest(self) -> global___CreateConfigurationCommitRequest: ...
    @property
    def retrieveConfigurationCommitRequest(self) -> global___RetrieveConfigurationCommitRequest: ...
    @property
    def executeDevelopmentComputeRequest(self) -> global___ExecuteDevelopmentComputeRequest: ...
    @property
    def generateMergeApprovalSignatureRequest(self) -> global___GenerateMergeApprovalSignatureRequest: ...
    @property
    def mergeConfigurationCommitRequest(self) -> global___MergeConfigurationCommitRequest: ...
    @property
    def retrieveConfigurationCommitApproversRequest(self) -> global___RetrieveConfigurationCommitApproversRequest: ...
    @property
    def casAuxiliaryStateRequest(self) -> global___CasAuxiliaryStateRequest: ...
    @property
    def readAuxiliaryStateRequest(self) -> global___ReadAuxiliaryStateRequest: ...
    @property
    def retrieveUsedAirlockQuotaRequest(self) -> global___RetrieveUsedAirlockQuotaRequest: ...
    @property
    def getResultsSizeRequest(self) -> global___GetResultsSizeRequest: ...
    @property
    def endorsementRequest(self) -> identity_endorsement_pb2.EndorsementRequest: ...
    def __init__(
        self,
        *,
        userAuth: global___UserAuth | None = ...,
        createDataRoomRequest: global___CreateDataRoomRequest | None = ...,
        retrieveDataRoomRequest: global___RetrieveDataRoomRequest | None = ...,
        retrieveCurrentDataRoomConfigurationRequest: global___RetrieveCurrentDataRoomConfigurationRequest | None = ...,
        retrieveDataRoomStatusRequest: global___RetrieveDataRoomStatusRequest | None = ...,
        updateDataRoomStatusRequest: global___UpdateDataRoomStatusRequest | None = ...,
        retrieveAuditLogRequest: global___RetrieveAuditLogRequest | None = ...,
        publishDatasetToDataRoomRequest: global___PublishDatasetToDataRoomRequest | None = ...,
        retrievePublishedDatasetsRequest: global___RetrievePublishedDatasetsRequest | None = ...,
        removePublishedDatasetRequest: global___RemovePublishedDatasetRequest | None = ...,
        executeComputeRequest: global___ExecuteComputeRequest | None = ...,
        jobStatusRequest: global___JobStatusRequest | None = ...,
        getResultsRequest: global___GetResultsRequest | None = ...,
        createConfigurationCommitRequest: global___CreateConfigurationCommitRequest | None = ...,
        retrieveConfigurationCommitRequest: global___RetrieveConfigurationCommitRequest | None = ...,
        executeDevelopmentComputeRequest: global___ExecuteDevelopmentComputeRequest | None = ...,
        generateMergeApprovalSignatureRequest: global___GenerateMergeApprovalSignatureRequest | None = ...,
        mergeConfigurationCommitRequest: global___MergeConfigurationCommitRequest | None = ...,
        retrieveConfigurationCommitApproversRequest: global___RetrieveConfigurationCommitApproversRequest | None = ...,
        casAuxiliaryStateRequest: global___CasAuxiliaryStateRequest | None = ...,
        readAuxiliaryStateRequest: global___ReadAuxiliaryStateRequest | None = ...,
        retrieveUsedAirlockQuotaRequest: global___RetrieveUsedAirlockQuotaRequest | None = ...,
        getResultsSizeRequest: global___GetResultsSizeRequest | None = ...,
        endorsementRequest: identity_endorsement_pb2.EndorsementRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["casAuxiliaryStateRequest", b"casAuxiliaryStateRequest", "createConfigurationCommitRequest", b"createConfigurationCommitRequest", "createDataRoomRequest", b"createDataRoomRequest", "endorsementRequest", b"endorsementRequest", "executeComputeRequest", b"executeComputeRequest", "executeDevelopmentComputeRequest", b"executeDevelopmentComputeRequest", "gcg_request", b"gcg_request", "generateMergeApprovalSignatureRequest", b"generateMergeApprovalSignatureRequest", "getResultsRequest", b"getResultsRequest", "getResultsSizeRequest", b"getResultsSizeRequest", "jobStatusRequest", b"jobStatusRequest", "mergeConfigurationCommitRequest", b"mergeConfigurationCommitRequest", "publishDatasetToDataRoomRequest", b"publishDatasetToDataRoomRequest", "readAuxiliaryStateRequest", b"readAuxiliaryStateRequest", "removePublishedDatasetRequest", b"removePublishedDatasetRequest", "retrieveAuditLogRequest", b"retrieveAuditLogRequest", "retrieveConfigurationCommitApproversRequest", b"retrieveConfigurationCommitApproversRequest", "retrieveConfigurationCommitRequest", b"retrieveConfigurationCommitRequest", "retrieveCurrentDataRoomConfigurationRequest", b"retrieveCurrentDataRoomConfigurationRequest", "retrieveDataRoomRequest", b"retrieveDataRoomRequest", "retrieveDataRoomStatusRequest", b"retrieveDataRoomStatusRequest", "retrievePublishedDatasetsRequest", b"retrievePublishedDatasetsRequest", "retrieveUsedAirlockQuotaRequest", b"retrieveUsedAirlockQuotaRequest", "updateDataRoomStatusRequest", b"updateDataRoomStatusRequest", "userAuth", b"userAuth"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["casAuxiliaryStateRequest", b"casAuxiliaryStateRequest", "createConfigurationCommitRequest", b"createConfigurationCommitRequest", "createDataRoomRequest", b"createDataRoomRequest", "endorsementRequest", b"endorsementRequest", "executeComputeRequest", b"executeComputeRequest", "executeDevelopmentComputeRequest", b"executeDevelopmentComputeRequest", "gcg_request", b"gcg_request", "generateMergeApprovalSignatureRequest", b"generateMergeApprovalSignatureRequest", "getResultsRequest", b"getResultsRequest", "getResultsSizeRequest", b"getResultsSizeRequest", "jobStatusRequest", b"jobStatusRequest", "mergeConfigurationCommitRequest", b"mergeConfigurationCommitRequest", "publishDatasetToDataRoomRequest", b"publishDatasetToDataRoomRequest", "readAuxiliaryStateRequest", b"readAuxiliaryStateRequest", "removePublishedDatasetRequest", b"removePublishedDatasetRequest", "retrieveAuditLogRequest", b"retrieveAuditLogRequest", "retrieveConfigurationCommitApproversRequest", b"retrieveConfigurationCommitApproversRequest", "retrieveConfigurationCommitRequest", b"retrieveConfigurationCommitRequest", "retrieveCurrentDataRoomConfigurationRequest", b"retrieveCurrentDataRoomConfigurationRequest", "retrieveDataRoomRequest", b"retrieveDataRoomRequest", "retrieveDataRoomStatusRequest", b"retrieveDataRoomStatusRequest", "retrievePublishedDatasetsRequest", b"retrievePublishedDatasetsRequest", "retrieveUsedAirlockQuotaRequest", b"retrieveUsedAirlockQuotaRequest", "updateDataRoomStatusRequest", b"updateDataRoomStatusRequest", "userAuth", b"userAuth"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["gcg_request", b"gcg_request"]) -> typing_extensions.Literal["createDataRoomRequest", "retrieveDataRoomRequest", "retrieveCurrentDataRoomConfigurationRequest", "retrieveDataRoomStatusRequest", "updateDataRoomStatusRequest", "retrieveAuditLogRequest", "publishDatasetToDataRoomRequest", "retrievePublishedDatasetsRequest", "removePublishedDatasetRequest", "executeComputeRequest", "jobStatusRequest", "getResultsRequest", "createConfigurationCommitRequest", "retrieveConfigurationCommitRequest", "executeDevelopmentComputeRequest", "generateMergeApprovalSignatureRequest", "mergeConfigurationCommitRequest", "retrieveConfigurationCommitApproversRequest", "casAuxiliaryStateRequest", "readAuxiliaryStateRequest", "retrieveUsedAirlockQuotaRequest", "getResultsSizeRequest", "endorsementRequest"] | None: ...

global___GcgRequest = GcgRequest

@typing_extensions.final
class GcgResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FAILURE_FIELD_NUMBER: builtins.int
    CREATEDATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVECURRENTDATAROOMCONFIGURATIONRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGRESPONSE_FIELD_NUMBER: builtins.int
    PUBLISHDATASETTODATAROOMRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSRESPONSE_FIELD_NUMBER: builtins.int
    REMOVEPUBLISHEDDATASETRESPONSE_FIELD_NUMBER: builtins.int
    EXECUTECOMPUTERESPONSE_FIELD_NUMBER: builtins.int
    JOBSTATUSRESPONSE_FIELD_NUMBER: builtins.int
    GETRESULTSRESPONSECHUNK_FIELD_NUMBER: builtins.int
    GETRESULTSRESPONSEFOOTER_FIELD_NUMBER: builtins.int
    CREATECONFIGURATIONCOMMITRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVECONFIGURATIONCOMMITRESPONSE_FIELD_NUMBER: builtins.int
    GENERATEMERGEAPPROVALSIGNATURERESPONSE_FIELD_NUMBER: builtins.int
    MERGECONFIGURATIONCOMMITRESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVECONFIGURATIONCOMMITAPPROVERSRESPONSE_FIELD_NUMBER: builtins.int
    CASAUXILIARYSTATERESPONSE_FIELD_NUMBER: builtins.int
    READAUXILIARYSTATERESPONSE_FIELD_NUMBER: builtins.int
    RETRIEVEUSEDAIRLOCKQUOTARESPONSE_FIELD_NUMBER: builtins.int
    GETRESULTSSIZERESPONSE_FIELD_NUMBER: builtins.int
    ENDORSEMENTRESPONSE_FIELD_NUMBER: builtins.int
    failure: builtins.str
    @property
    def createDataRoomResponse(self) -> global___CreateDataRoomResponse: ...
    @property
    def retrieveDataRoomResponse(self) -> global___RetrieveDataRoomResponse: ...
    @property
    def retrieveCurrentDataRoomConfigurationResponse(self) -> global___RetrieveCurrentDataRoomConfigurationResponse: ...
    @property
    def retrieveDataRoomStatusResponse(self) -> global___RetrieveDataRoomStatusResponse: ...
    @property
    def updateDataRoomStatusResponse(self) -> global___UpdateDataRoomStatusResponse: ...
    @property
    def retrieveAuditLogResponse(self) -> global___RetrieveAuditLogResponse: ...
    @property
    def publishDatasetToDataRoomResponse(self) -> global___PublishDatasetToDataRoomResponse: ...
    @property
    def retrievePublishedDatasetsResponse(self) -> global___RetrievePublishedDatasetsResponse: ...
    @property
    def removePublishedDatasetResponse(self) -> global___RemovePublishedDatasetResponse: ...
    @property
    def executeComputeResponse(self) -> global___ExecuteComputeResponse: ...
    @property
    def jobStatusResponse(self) -> global___JobStatusResponse: ...
    @property
    def getResultsResponseChunk(self) -> global___GetResultsResponseChunk: ...
    @property
    def getResultsResponseFooter(self) -> global___GetResultsResponseFooter: ...
    @property
    def createConfigurationCommitResponse(self) -> global___CreateConfigurationCommitResponse: ...
    @property
    def retrieveConfigurationCommitResponse(self) -> global___RetrieveConfigurationCommitResponse: ...
    @property
    def generateMergeApprovalSignatureResponse(self) -> global___GenerateMergeApprovalSignatureResponse: ...
    @property
    def mergeConfigurationCommitResponse(self) -> global___MergeConfigurationCommitResponse: ...
    @property
    def retrieveConfigurationCommitApproversResponse(self) -> global___RetrieveConfigurationCommitApproversResponse: ...
    @property
    def casAuxiliaryStateResponse(self) -> global___CasAuxiliaryStateResponse: ...
    @property
    def readAuxiliaryStateResponse(self) -> global___ReadAuxiliaryStateResponse: ...
    @property
    def retrieveUsedAirlockQuotaResponse(self) -> global___RetrieveUsedAirlockQuotaResponse: ...
    @property
    def getResultsSizeResponse(self) -> global___GetResultsSizeResponse: ...
    @property
    def endorsementResponse(self) -> identity_endorsement_pb2.EndorsementResponse: ...
    def __init__(
        self,
        *,
        failure: builtins.str = ...,
        createDataRoomResponse: global___CreateDataRoomResponse | None = ...,
        retrieveDataRoomResponse: global___RetrieveDataRoomResponse | None = ...,
        retrieveCurrentDataRoomConfigurationResponse: global___RetrieveCurrentDataRoomConfigurationResponse | None = ...,
        retrieveDataRoomStatusResponse: global___RetrieveDataRoomStatusResponse | None = ...,
        updateDataRoomStatusResponse: global___UpdateDataRoomStatusResponse | None = ...,
        retrieveAuditLogResponse: global___RetrieveAuditLogResponse | None = ...,
        publishDatasetToDataRoomResponse: global___PublishDatasetToDataRoomResponse | None = ...,
        retrievePublishedDatasetsResponse: global___RetrievePublishedDatasetsResponse | None = ...,
        removePublishedDatasetResponse: global___RemovePublishedDatasetResponse | None = ...,
        executeComputeResponse: global___ExecuteComputeResponse | None = ...,
        jobStatusResponse: global___JobStatusResponse | None = ...,
        getResultsResponseChunk: global___GetResultsResponseChunk | None = ...,
        getResultsResponseFooter: global___GetResultsResponseFooter | None = ...,
        createConfigurationCommitResponse: global___CreateConfigurationCommitResponse | None = ...,
        retrieveConfigurationCommitResponse: global___RetrieveConfigurationCommitResponse | None = ...,
        generateMergeApprovalSignatureResponse: global___GenerateMergeApprovalSignatureResponse | None = ...,
        mergeConfigurationCommitResponse: global___MergeConfigurationCommitResponse | None = ...,
        retrieveConfigurationCommitApproversResponse: global___RetrieveConfigurationCommitApproversResponse | None = ...,
        casAuxiliaryStateResponse: global___CasAuxiliaryStateResponse | None = ...,
        readAuxiliaryStateResponse: global___ReadAuxiliaryStateResponse | None = ...,
        retrieveUsedAirlockQuotaResponse: global___RetrieveUsedAirlockQuotaResponse | None = ...,
        getResultsSizeResponse: global___GetResultsSizeResponse | None = ...,
        endorsementResponse: identity_endorsement_pb2.EndorsementResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["casAuxiliaryStateResponse", b"casAuxiliaryStateResponse", "createConfigurationCommitResponse", b"createConfigurationCommitResponse", "createDataRoomResponse", b"createDataRoomResponse", "endorsementResponse", b"endorsementResponse", "executeComputeResponse", b"executeComputeResponse", "failure", b"failure", "gcg_response", b"gcg_response", "generateMergeApprovalSignatureResponse", b"generateMergeApprovalSignatureResponse", "getResultsResponseChunk", b"getResultsResponseChunk", "getResultsResponseFooter", b"getResultsResponseFooter", "getResultsSizeResponse", b"getResultsSizeResponse", "jobStatusResponse", b"jobStatusResponse", "mergeConfigurationCommitResponse", b"mergeConfigurationCommitResponse", "publishDatasetToDataRoomResponse", b"publishDatasetToDataRoomResponse", "readAuxiliaryStateResponse", b"readAuxiliaryStateResponse", "removePublishedDatasetResponse", b"removePublishedDatasetResponse", "retrieveAuditLogResponse", b"retrieveAuditLogResponse", "retrieveConfigurationCommitApproversResponse", b"retrieveConfigurationCommitApproversResponse", "retrieveConfigurationCommitResponse", b"retrieveConfigurationCommitResponse", "retrieveCurrentDataRoomConfigurationResponse", b"retrieveCurrentDataRoomConfigurationResponse", "retrieveDataRoomResponse", b"retrieveDataRoomResponse", "retrieveDataRoomStatusResponse", b"retrieveDataRoomStatusResponse", "retrievePublishedDatasetsResponse", b"retrievePublishedDatasetsResponse", "retrieveUsedAirlockQuotaResponse", b"retrieveUsedAirlockQuotaResponse", "updateDataRoomStatusResponse", b"updateDataRoomStatusResponse"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["casAuxiliaryStateResponse", b"casAuxiliaryStateResponse", "createConfigurationCommitResponse", b"createConfigurationCommitResponse", "createDataRoomResponse", b"createDataRoomResponse", "endorsementResponse", b"endorsementResponse", "executeComputeResponse", b"executeComputeResponse", "failure", b"failure", "gcg_response", b"gcg_response", "generateMergeApprovalSignatureResponse", b"generateMergeApprovalSignatureResponse", "getResultsResponseChunk", b"getResultsResponseChunk", "getResultsResponseFooter", b"getResultsResponseFooter", "getResultsSizeResponse", b"getResultsSizeResponse", "jobStatusResponse", b"jobStatusResponse", "mergeConfigurationCommitResponse", b"mergeConfigurationCommitResponse", "publishDatasetToDataRoomResponse", b"publishDatasetToDataRoomResponse", "readAuxiliaryStateResponse", b"readAuxiliaryStateResponse", "removePublishedDatasetResponse", b"removePublishedDatasetResponse", "retrieveAuditLogResponse", b"retrieveAuditLogResponse", "retrieveConfigurationCommitApproversResponse", b"retrieveConfigurationCommitApproversResponse", "retrieveConfigurationCommitResponse", b"retrieveConfigurationCommitResponse", "retrieveCurrentDataRoomConfigurationResponse", b"retrieveCurrentDataRoomConfigurationResponse", "retrieveDataRoomResponse", b"retrieveDataRoomResponse", "retrieveDataRoomStatusResponse", b"retrieveDataRoomStatusResponse", "retrievePublishedDatasetsResponse", b"retrievePublishedDatasetsResponse", "retrieveUsedAirlockQuotaResponse", b"retrieveUsedAirlockQuotaResponse", "updateDataRoomStatusResponse", b"updateDataRoomStatusResponse"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["gcg_response", b"gcg_response"]) -> typing_extensions.Literal["failure", "createDataRoomResponse", "retrieveDataRoomResponse", "retrieveCurrentDataRoomConfigurationResponse", "retrieveDataRoomStatusResponse", "updateDataRoomStatusResponse", "retrieveAuditLogResponse", "publishDatasetToDataRoomResponse", "retrievePublishedDatasetsResponse", "removePublishedDatasetResponse", "executeComputeResponse", "jobStatusResponse", "getResultsResponseChunk", "getResultsResponseFooter", "createConfigurationCommitResponse", "retrieveConfigurationCommitResponse", "generateMergeApprovalSignatureResponse", "mergeConfigurationCommitResponse", "retrieveConfigurationCommitApproversResponse", "casAuxiliaryStateResponse", "readAuxiliaryStateResponse", "retrieveUsedAirlockQuotaResponse", "getResultsSizeResponse", "endorsementResponse"] | None: ...

global___GcgResponse = GcgResponse

@typing_extensions.final
class UserAuth(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PKI_FIELD_NUMBER: builtins.int
    ENCLAVEENDORSEMENTS_FIELD_NUMBER: builtins.int
    @property
    def pki(self) -> global___Pki: ...
    @property
    def enclaveEndorsements(self) -> identity_endorsement_pb2.EnclaveEndorsements: ...
    def __init__(
        self,
        *,
        pki: global___Pki | None = ...,
        enclaveEndorsements: identity_endorsement_pb2.EnclaveEndorsements | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["enclaveEndorsements", b"enclaveEndorsements", "pki", b"pki"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["enclaveEndorsements", b"enclaveEndorsements", "pki", b"pki"]) -> None: ...

global___UserAuth = UserAuth

@typing_extensions.final
class Pki(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CERTCHAINPEM_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    IDMAC_FIELD_NUMBER: builtins.int
    certChainPem: builtins.bytes
    signature: builtins.bytes
    idMac: builtins.bytes
    def __init__(
        self,
        *,
        certChainPem: builtins.bytes = ...,
        signature: builtins.bytes = ...,
        idMac: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["certChainPem", b"certChainPem", "idMac", b"idMac", "signature", b"signature"]) -> None: ...

global___Pki = Pki

@typing_extensions.final
class CreateDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    HIGHLEVELREPRESENTATION_FIELD_NUMBER: builtins.int
    DATAROOMMETADATA_FIELD_NUMBER: builtins.int
    @property
    def dataRoom(self) -> data_room_pb2.DataRoom: ...
    highLevelRepresentation: builtins.bytes
    dataRoomMetadata: builtins.bytes
    def __init__(
        self,
        *,
        dataRoom: data_room_pb2.DataRoom | None = ...,
        highLevelRepresentation: builtins.bytes | None = ...,
        dataRoomMetadata: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "_highLevelRepresentation", b"_highLevelRepresentation", "dataRoom", b"dataRoom", "dataRoomMetadata", b"dataRoomMetadata", "highLevelRepresentation", b"highLevelRepresentation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "_highLevelRepresentation", b"_highLevelRepresentation", "dataRoom", b"dataRoom", "dataRoomMetadata", b"dataRoomMetadata", "highLevelRepresentation", b"highLevelRepresentation"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata"]) -> typing_extensions.Literal["dataRoomMetadata"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation"]) -> typing_extensions.Literal["highLevelRepresentation"] | None: ...

global___CreateDataRoomRequest = CreateDataRoomRequest

@typing_extensions.final
class CreateDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMVALIDATIONERROR_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    @property
    def dataRoomValidationError(self) -> global___DataRoomValidationError: ...
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        dataRoomValidationError: global___DataRoomValidationError | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["create_data_room_response", b"create_data_room_response", "dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["create_data_room_response", b"create_data_room_response", "dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["create_data_room_response", b"create_data_room_response"]) -> typing_extensions.Literal["dataRoomId", "dataRoomValidationError"] | None: ...

global___CreateDataRoomResponse = CreateDataRoomResponse

@typing_extensions.final
class DataRoomValidationError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    PERMISSIONINDEX_FIELD_NUMBER: builtins.int
    COMPUTENODEID_FIELD_NUMBER: builtins.int
    USERPERMISSIONID_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONID_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODID_FIELD_NUMBER: builtins.int
    message: builtins.str
    permissionIndex: builtins.int
    computeNodeId: builtins.str
    userPermissionId: builtins.str
    attestationSpecificationId: builtins.str
    authenticationMethodId: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        permissionIndex: builtins.int | None = ...,
        computeNodeId: builtins.str | None = ...,
        userPermissionId: builtins.str | None = ...,
        attestationSpecificationId: builtins.str | None = ...,
        authenticationMethodId: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "message", b"message", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId"]) -> typing_extensions.Literal["attestationSpecificationId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_authenticationMethodId", b"_authenticationMethodId"]) -> typing_extensions.Literal["authenticationMethodId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_computeNodeId", b"_computeNodeId"]) -> typing_extensions.Literal["computeNodeId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_permissionIndex", b"_permissionIndex"]) -> typing_extensions.Literal["permissionIndex"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_userPermissionId", b"_userPermissionId"]) -> typing_extensions.Literal["userPermissionId"] | None: ...

global___DataRoomValidationError = DataRoomValidationError

@typing_extensions.final
class PublishDatasetToDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATASETHASH_FIELD_NUMBER: builtins.int
    DATAROOMID_FIELD_NUMBER: builtins.int
    LEAFID_FIELD_NUMBER: builtins.int
    ENCRYPTIONKEY_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    datasetHash: builtins.bytes
    dataRoomId: builtins.bytes
    leafId: builtins.str
    encryptionKey: builtins.bytes
    scope: builtins.bytes
    def __init__(
        self,
        *,
        datasetHash: builtins.bytes = ...,
        dataRoomId: builtins.bytes = ...,
        leafId: builtins.str = ...,
        encryptionKey: builtins.bytes = ...,
        scope: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "datasetHash", b"datasetHash", "encryptionKey", b"encryptionKey", "leafId", b"leafId", "scope", b"scope"]) -> None: ...

global___PublishDatasetToDataRoomRequest = PublishDatasetToDataRoomRequest

@typing_extensions.final
class PublishDatasetToDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___PublishDatasetToDataRoomResponse = PublishDatasetToDataRoomResponse

@typing_extensions.final
class ExecuteComputeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ParametersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing_extensions.final
    class TestDatasetsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___TestDataset: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___TestDataset | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    DATAROOMID_FIELD_NUMBER: builtins.int
    COMPUTENODEIDS_FIELD_NUMBER: builtins.int
    ISDRYRUN_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    PARAMETERS_FIELD_NUMBER: builtins.int
    TESTDATASETS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    @property
    def computeNodeIds(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    isDryRun: builtins.bool
    scope: builtins.bytes
    @property
    def parameters(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def testDatasets(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___TestDataset]: ...
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        computeNodeIds: collections.abc.Iterable[builtins.str] | None = ...,
        isDryRun: builtins.bool = ...,
        scope: builtins.bytes = ...,
        parameters: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        testDatasets: collections.abc.Mapping[builtins.str, global___TestDataset] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["computeNodeIds", b"computeNodeIds", "dataRoomId", b"dataRoomId", "isDryRun", b"isDryRun", "parameters", b"parameters", "scope", b"scope", "testDatasets", b"testDatasets"]) -> None: ...

global___ExecuteComputeRequest = ExecuteComputeRequest

@typing_extensions.final
class ExecuteDevelopmentComputeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ParametersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing_extensions.final
    class TestDatasetsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___TestDataset: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___TestDataset | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    CONFIGURATIONCOMMITID_FIELD_NUMBER: builtins.int
    COMPUTENODEIDS_FIELD_NUMBER: builtins.int
    ISDRYRUN_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    PARAMETERS_FIELD_NUMBER: builtins.int
    TESTDATASETS_FIELD_NUMBER: builtins.int
    configurationCommitId: builtins.bytes
    @property
    def computeNodeIds(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    isDryRun: builtins.bool
    scope: builtins.bytes
    @property
    def parameters(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def testDatasets(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___TestDataset]: ...
    def __init__(
        self,
        *,
        configurationCommitId: builtins.bytes = ...,
        computeNodeIds: collections.abc.Iterable[builtins.str] | None = ...,
        isDryRun: builtins.bool = ...,
        scope: builtins.bytes = ...,
        parameters: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        testDatasets: collections.abc.Mapping[builtins.str, global___TestDataset] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["computeNodeIds", b"computeNodeIds", "configurationCommitId", b"configurationCommitId", "isDryRun", b"isDryRun", "parameters", b"parameters", "scope", b"scope", "testDatasets", b"testDatasets"]) -> None: ...

global___ExecuteDevelopmentComputeRequest = ExecuteDevelopmentComputeRequest

@typing_extensions.final
class TestDataset(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENCRYPTIONKEY_FIELD_NUMBER: builtins.int
    MANIFESTHASH_FIELD_NUMBER: builtins.int
    encryptionKey: builtins.bytes
    manifestHash: builtins.bytes
    def __init__(
        self,
        *,
        encryptionKey: builtins.bytes = ...,
        manifestHash: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["encryptionKey", b"encryptionKey", "manifestHash", b"manifestHash"]) -> None: ...

global___TestDataset = TestDataset

@typing_extensions.final
class ExecuteComputeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes
    def __init__(
        self,
        *,
        jobId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["jobId", b"jobId"]) -> None: ...

global___ExecuteComputeResponse = ExecuteComputeResponse

@typing_extensions.final
class JobStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes
    def __init__(
        self,
        *,
        jobId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["jobId", b"jobId"]) -> None: ...

global___JobStatusRequest = JobStatusRequest

@typing_extensions.final
class JobStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMPLETECOMPUTENODEIDS_FIELD_NUMBER: builtins.int
    @property
    def completeComputeNodeIds(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        completeComputeNodeIds: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["completeComputeNodeIds", b"completeComputeNodeIds"]) -> None: ...

global___JobStatusResponse = JobStatusResponse

@typing_extensions.final
class GetResultsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBID_FIELD_NUMBER: builtins.int
    COMPUTENODEID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes
    computeNodeId: builtins.str
    def __init__(
        self,
        *,
        jobId: builtins.bytes = ...,
        computeNodeId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["computeNodeId", b"computeNodeId", "jobId", b"jobId"]) -> None: ...

global___GetResultsRequest = GetResultsRequest

@typing_extensions.final
class GetResultsSizeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOBID_FIELD_NUMBER: builtins.int
    COMPUTENODEID_FIELD_NUMBER: builtins.int
    jobId: builtins.bytes
    computeNodeId: builtins.str
    def __init__(
        self,
        *,
        jobId: builtins.bytes = ...,
        computeNodeId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["computeNodeId", b"computeNodeId", "jobId", b"jobId"]) -> None: ...

global___GetResultsSizeRequest = GetResultsSizeRequest

@typing_extensions.final
class GetResultsResponseChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data", b"data"]) -> None: ...

global___GetResultsResponseChunk = GetResultsResponseChunk

@typing_extensions.final
class GetResultsResponseFooter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetResultsResponseFooter = GetResultsResponseFooter

@typing_extensions.final
class RetrieveDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveDataRoomRequest = RetrieveDataRoomRequest

@typing_extensions.final
class RetrieveDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    COMMITS_FIELD_NUMBER: builtins.int
    HIGHLEVELREPRESENTATION_FIELD_NUMBER: builtins.int
    @property
    def dataRoom(self) -> data_room_pb2.DataRoom: ...
    @property
    def commits(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[data_room_pb2.ConfigurationCommit]: ...
    highLevelRepresentation: builtins.bytes
    def __init__(
        self,
        *,
        dataRoom: data_room_pb2.DataRoom | None = ...,
        commits: collections.abc.Iterable[data_room_pb2.ConfigurationCommit] | None = ...,
        highLevelRepresentation: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "dataRoom", b"dataRoom", "highLevelRepresentation", b"highLevelRepresentation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "commits", b"commits", "dataRoom", b"dataRoom", "highLevelRepresentation", b"highLevelRepresentation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation"]) -> typing_extensions.Literal["highLevelRepresentation"] | None: ...

global___RetrieveDataRoomResponse = RetrieveDataRoomResponse

@typing_extensions.final
class RetrieveAuditLogRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveAuditLogRequest = RetrieveAuditLogRequest

@typing_extensions.final
class RetrieveAuditLogResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOG_FIELD_NUMBER: builtins.int
    log: builtins.bytes
    def __init__(
        self,
        *,
        log: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["log", b"log"]) -> None: ...

global___RetrieveAuditLogResponse = RetrieveAuditLogResponse

@typing_extensions.final
class RetrieveDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveDataRoomStatusRequest = RetrieveDataRoomStatusRequest

@typing_extensions.final
class RetrieveDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATUS_FIELD_NUMBER: builtins.int
    status: global___DataRoomStatus.ValueType
    def __init__(
        self,
        *,
        status: global___DataRoomStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["status", b"status"]) -> None: ...

global___RetrieveDataRoomStatusResponse = RetrieveDataRoomStatusResponse

@typing_extensions.final
class UpdateDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    status: global___DataRoomStatus.ValueType
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        status: global___DataRoomStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "status", b"status"]) -> None: ...

global___UpdateDataRoomStatusRequest = UpdateDataRoomStatusRequest

@typing_extensions.final
class UpdateDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateDataRoomStatusResponse = UpdateDataRoomStatusResponse

@typing_extensions.final
class RetrievePublishedDatasetsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrievePublishedDatasetsRequest = RetrievePublishedDatasetsRequest

@typing_extensions.final
class PublishedDataset(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEAFID_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    DATASETHASH_FIELD_NUMBER: builtins.int
    leafId: builtins.str
    user: builtins.str
    timestamp: builtins.int
    datasetHash: builtins.bytes
    def __init__(
        self,
        *,
        leafId: builtins.str = ...,
        user: builtins.str = ...,
        timestamp: builtins.int = ...,
        datasetHash: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["datasetHash", b"datasetHash", "leafId", b"leafId", "timestamp", b"timestamp", "user", b"user"]) -> None: ...

global___PublishedDataset = PublishedDataset

@typing_extensions.final
class RetrievePublishedDatasetsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PUBLISHEDDATASETS_FIELD_NUMBER: builtins.int
    @property
    def publishedDatasets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PublishedDataset]: ...
    def __init__(
        self,
        *,
        publishedDatasets: collections.abc.Iterable[global___PublishedDataset] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["publishedDatasets", b"publishedDatasets"]) -> None: ...

global___RetrievePublishedDatasetsResponse = RetrievePublishedDatasetsResponse

@typing_extensions.final
class RemovePublishedDatasetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    LEAFID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    leafId: builtins.str
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        leafId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "leafId", b"leafId"]) -> None: ...

global___RemovePublishedDatasetRequest = RemovePublishedDatasetRequest

@typing_extensions.final
class RemovePublishedDatasetResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RemovePublishedDatasetResponse = RemovePublishedDatasetResponse

@typing_extensions.final
class CreateConfigurationCommitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMIT_FIELD_NUMBER: builtins.int
    HIGHLEVELREPRESENTATION_FIELD_NUMBER: builtins.int
    @property
    def commit(self) -> data_room_pb2.ConfigurationCommit: ...
    highLevelRepresentation: builtins.bytes
    def __init__(
        self,
        *,
        commit: data_room_pb2.ConfigurationCommit | None = ...,
        highLevelRepresentation: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "commit", b"commit", "highLevelRepresentation", b"highLevelRepresentation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "commit", b"commit", "highLevelRepresentation", b"highLevelRepresentation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation"]) -> typing_extensions.Literal["highLevelRepresentation"] | None: ...

global___CreateConfigurationCommitRequest = CreateConfigurationCommitRequest

@typing_extensions.final
class CreateConfigurationCommitResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMITID_FIELD_NUMBER: builtins.int
    commitId: builtins.bytes
    def __init__(
        self,
        *,
        commitId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitId", b"commitId"]) -> None: ...

global___CreateConfigurationCommitResponse = CreateConfigurationCommitResponse

@typing_extensions.final
class GenerateMergeApprovalSignatureRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMITID_FIELD_NUMBER: builtins.int
    commitId: builtins.bytes
    def __init__(
        self,
        *,
        commitId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitId", b"commitId"]) -> None: ...

global___GenerateMergeApprovalSignatureRequest = GenerateMergeApprovalSignatureRequest

@typing_extensions.final
class GenerateMergeApprovalSignatureResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SIGNATURE_FIELD_NUMBER: builtins.int
    signature: builtins.bytes
    def __init__(
        self,
        *,
        signature: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["signature", b"signature"]) -> None: ...

global___GenerateMergeApprovalSignatureResponse = GenerateMergeApprovalSignatureResponse

@typing_extensions.final
class MergeConfigurationCommitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ApprovalSignaturesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.bytes
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.bytes = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    COMMITID_FIELD_NUMBER: builtins.int
    APPROVALSIGNATURES_FIELD_NUMBER: builtins.int
    NEWDATAROOMHIGHLEVELREPRESENTATION_FIELD_NUMBER: builtins.int
    commitId: builtins.bytes
    @property
    def approvalSignatures(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.bytes]: ...
    newDataRoomHighLevelRepresentation: builtins.bytes
    def __init__(
        self,
        *,
        commitId: builtins.bytes = ...,
        approvalSignatures: collections.abc.Mapping[builtins.str, builtins.bytes] | None = ...,
        newDataRoomHighLevelRepresentation: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_newDataRoomHighLevelRepresentation", b"_newDataRoomHighLevelRepresentation", "newDataRoomHighLevelRepresentation", b"newDataRoomHighLevelRepresentation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_newDataRoomHighLevelRepresentation", b"_newDataRoomHighLevelRepresentation", "approvalSignatures", b"approvalSignatures", "commitId", b"commitId", "newDataRoomHighLevelRepresentation", b"newDataRoomHighLevelRepresentation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_newDataRoomHighLevelRepresentation", b"_newDataRoomHighLevelRepresentation"]) -> typing_extensions.Literal["newDataRoomHighLevelRepresentation"] | None: ...

global___MergeConfigurationCommitRequest = MergeConfigurationCommitRequest

@typing_extensions.final
class RetrieveCurrentDataRoomConfigurationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveCurrentDataRoomConfigurationRequest = RetrieveCurrentDataRoomConfigurationRequest

@typing_extensions.final
class RetrieveCurrentDataRoomConfigurationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONFIGURATION_FIELD_NUMBER: builtins.int
    PIN_FIELD_NUMBER: builtins.int
    @property
    def configuration(self) -> data_room_pb2.DataRoomConfiguration: ...
    pin: builtins.bytes
    def __init__(
        self,
        *,
        configuration: data_room_pb2.DataRoomConfiguration | None = ...,
        pin: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["configuration", b"configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "pin", b"pin"]) -> None: ...

global___RetrieveCurrentDataRoomConfigurationResponse = RetrieveCurrentDataRoomConfigurationResponse

@typing_extensions.final
class RetrieveConfigurationCommitApproversRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMITID_FIELD_NUMBER: builtins.int
    commitId: builtins.bytes
    def __init__(
        self,
        *,
        commitId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitId", b"commitId"]) -> None: ...

global___RetrieveConfigurationCommitApproversRequest = RetrieveConfigurationCommitApproversRequest

@typing_extensions.final
class RetrieveConfigurationCommitApproversResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APPROVERS_FIELD_NUMBER: builtins.int
    @property
    def approvers(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        approvers: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["approvers", b"approvers"]) -> None: ...

global___RetrieveConfigurationCommitApproversResponse = RetrieveConfigurationCommitApproversResponse

@typing_extensions.final
class RetrieveConfigurationCommitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMITID_FIELD_NUMBER: builtins.int
    commitId: builtins.bytes
    def __init__(
        self,
        *,
        commitId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commitId", b"commitId"]) -> None: ...

global___RetrieveConfigurationCommitRequest = RetrieveConfigurationCommitRequest

@typing_extensions.final
class RetrieveConfigurationCommitResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMIT_FIELD_NUMBER: builtins.int
    HIGHLEVELREPRESENTATION_FIELD_NUMBER: builtins.int
    @property
    def commit(self) -> data_room_pb2.ConfigurationCommit: ...
    highLevelRepresentation: builtins.bytes
    def __init__(
        self,
        *,
        commit: data_room_pb2.ConfigurationCommit | None = ...,
        highLevelRepresentation: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "commit", b"commit", "highLevelRepresentation", b"highLevelRepresentation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation", "commit", b"commit", "highLevelRepresentation", b"highLevelRepresentation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_highLevelRepresentation", b"_highLevelRepresentation"]) -> typing_extensions.Literal["highLevelRepresentation"] | None: ...

global___RetrieveConfigurationCommitResponse = RetrieveConfigurationCommitResponse

@typing_extensions.final
class CasAuxiliaryStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    index: builtins.int
    """The index indicates what the client thinks the state is. The CAS operation will only succeed if the index
    matches what's stored in the enclave. The index is 0 iff the value doesn't exist/was deleted.
    """
    value: builtins.bytes
    """If null it will try to delete value."""
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        index: builtins.int = ...,
        value: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_value", b"_value", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_value", b"_value", "dataRoomId", b"dataRoomId", "index", b"index", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

global___CasAuxiliaryStateRequest = CasAuxiliaryStateRequest

@typing_extensions.final
class CasAuxiliaryStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    success: builtins.bool
    index: builtins.int
    """The index at the end of the operation, 0 if delete was successful. If success=false, the client should use this
    index and retry the operation, possibly modified based on the returned value.
    Example: User A is racing on two clients X and Y to add numbers 1 and 2 to an initially empty list.
      1. index=0 list=[]: initial state
      2. index=1 list=[1]: User A on client X does CAS(index=0, value=[1])
      3. index=1 list=[1]: User A on client Y does CAS(index=0, value=[2]) => success=false, index=1, value=[1]
      4. index=2 list=[1,2]: User A on client Y retries with CAS(index=1, value=[1,2])
    """
    value: builtins.bytes
    """The value at the end of the operation."""
    def __init__(
        self,
        *,
        success: builtins.bool = ...,
        index: builtins.int = ...,
        value: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_value", b"_value", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_value", b"_value", "index", b"index", "success", b"success", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

global___CasAuxiliaryStateResponse = CasAuxiliaryStateResponse

@typing_extensions.final
class ReadAuxiliaryStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___ReadAuxiliaryStateRequest = ReadAuxiliaryStateRequest

@typing_extensions.final
class ReadAuxiliaryStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AuxiliaryStateValue]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___AuxiliaryStateValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["values", b"values"]) -> None: ...

global___ReadAuxiliaryStateResponse = ReadAuxiliaryStateResponse

@typing_extensions.final
class AuxiliaryStateValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USER_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    user: builtins.str
    index: builtins.int
    value: builtins.bytes
    def __init__(
        self,
        *,
        user: builtins.str = ...,
        index: builtins.int = ...,
        value: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["index", b"index", "user", b"user", "value", b"value"]) -> None: ...

global___AuxiliaryStateValue = AuxiliaryStateValue

@typing_extensions.final
class MergeConfigurationCommitResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MergeConfigurationCommitResponse = MergeConfigurationCommitResponse

@typing_extensions.final
class DriverTaskConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NOOP_FIELD_NUMBER: builtins.int
    STATICCONTENT_FIELD_NUMBER: builtins.int
    @property
    def noop(self) -> global___NoopConfig: ...
    @property
    def staticContent(self) -> global___StaticContentConfig: ...
    def __init__(
        self,
        *,
        noop: global___NoopConfig | None = ...,
        staticContent: global___StaticContentConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["driver_task_config", b"driver_task_config", "noop", b"noop", "staticContent", b"staticContent"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["driver_task_config", b"driver_task_config", "noop", b"noop", "staticContent", b"staticContent"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["driver_task_config", b"driver_task_config"]) -> typing_extensions.Literal["noop", "staticContent"] | None: ...

global___DriverTaskConfig = DriverTaskConfig

@typing_extensions.final
class NoopConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___NoopConfig = NoopConfig

@typing_extensions.final
class StaticContentConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["content", b"content"]) -> None: ...

global___StaticContentConfig = StaticContentConfig

@typing_extensions.final
class RetrieveUsedAirlockQuotaRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveUsedAirlockQuotaRequest = RetrieveUsedAirlockQuotaRequest

@typing_extensions.final
class RetrieveUsedAirlockQuotaResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AIRLOCKQUOTAS_FIELD_NUMBER: builtins.int
    @property
    def airlockQuotas(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AirlockQuotaInfo]: ...
    def __init__(
        self,
        *,
        airlockQuotas: collections.abc.Iterable[global___AirlockQuotaInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["airlockQuotas", b"airlockQuotas"]) -> None: ...

global___RetrieveUsedAirlockQuotaResponse = RetrieveUsedAirlockQuotaResponse

@typing_extensions.final
class AirlockQuotaInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AIRLOCKNODEID_FIELD_NUMBER: builtins.int
    QUOTABYTES_FIELD_NUMBER: builtins.int
    USEDQUOTABYTES_FIELD_NUMBER: builtins.int
    airlockNodeId: builtins.str
    quotaBytes: builtins.int
    usedQuotaBytes: builtins.int
    def __init__(
        self,
        *,
        airlockNodeId: builtins.str = ...,
        quotaBytes: builtins.int = ...,
        usedQuotaBytes: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["airlockNodeId", b"airlockNodeId", "quotaBytes", b"quotaBytes", "usedQuotaBytes", b"usedQuotaBytes"]) -> None: ...

global___AirlockQuotaInfo = AirlockQuotaInfo

@typing_extensions.final
class GetResultsSizeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SIZEBYTES_FIELD_NUMBER: builtins.int
    sizeBytes: builtins.int
    def __init__(
        self,
        *,
        sizeBytes: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sizeBytes", b"sizeBytes"]) -> None: ...

global___GetResultsSizeResponse = GetResultsSizeResponse



import io

from typing import Optional

# from ..session import Session
from ..storage import Key
from ..keychain import Keychain, KeychainEntry
from .builder import AnalyticsDcrBuilder, ParticipantPermission

from .python_compute_nodes import PythonComputeNode, PythonComputeNodeDefinition
from .r_compute_nodes import RComputeNode, RComputeNodeDefinition
from .table_data_nodes import (
    TableDataNode,
    Column,
    FormatType,
    TableDataNodeDefinition,
    PrimitiveType,
)
from .sql_compute_nodes import (
    SqlComputeNode,
    SqlComputeNodeDefinition,
)
from .sqlite_compute_nodes import (
    SqliteComputeNode,
    SqliteComputeNodeDefinition,
)
from .raw_data_nodes import (
    RawDataNode,
    RawDataNodeDefinition,
)
from .s3_sink_compute_nodes import (
    S3SinkComputeNode,
    S3Provider,
    S3SinkComputeNodeDefinition,
)
from .matching_compute_nodes import (
    MatchingComputeNode,
    MatchingComputeNodeDefinition,
    MatchingComputeNodeConfig,
)
from .preview_compute_nodes import PreviewComputeNode, PreviewComputeNodeDefinition
from .synthetic_compute_nodes import (
    SyntheticDataComputeNodeDefinition,
    SyntheticDataComputeNode,
    SyntheticNodeColumn,
    MaskType,
)
from .script import Script, ScriptingLanguage, PythonScript, RScript, FileContent
from .analytics_dcr import AnalyticsDcr, AnalyticsDcrDefinition
from .version import DATA_SCIENCE_DCR_SUPPORTED_VERSION
from .node_definitions import *
from .sql_helper import read_input_csv_file


__pdoc__ = {
    "builder": False,
    "commits": False,
    "compute_nodes": False,
    "data_nodes": False,
    "data_science_dcr": False,
    "existing_builder": False,
    "high_level_node": False,
    "script": False,
    "version": False,
    "analytics_dcr": False,
    "matching_compute_nodes": False,
    "node_definitions": False,
    "preview_compute_nodes": False,
    "python_compute_nodes": False,
    "r_compute_nodes": False,
    "raw_data_nodes": False,
    "s3_sink_compute_nodes": False,
    "sql_compute_nodes": False,
    "sql_helper": False,
    "sqlite_compute_nodes": False,
    "synthetic_compute_nodes": False,
    "table_data_nodes": False,
}

__docformat__ = "restructuredtext"

__all__ = [
    # DCR
    "AnalyticsDcr",
    "AnalyticsDcrDefinition",
    "AnalyticsDcrBuilder",
    # Compute nodes
    "PythonScript",
    "PythonComputeNodeDefinition",
    "RScript",
    "RComputeNodeDefinition",
    "PythonComputeNode",
    "PythonComputeNodeDefinition",
    "RComputeNode",
    "RComputeNodeDefinition",
    "SqlComputeNode",
    "SqlComputeNodeDefinition",
    "SqliteComputeNode",
    "SqliteComputeNodeDefinition",
    "S3SinkComputeNode",
    "S3SinkComputeNodeDefinition",
    "S3Provider",
    "MatchingComputeNode",
    "MatchingComputeNodeDefinition",
    "MatchingComputeNodeConfig",
    "MatchingComputeNodeDefinition",
    "SyntheticDataComputeNode",
    "SyntheticDataComputeNodeDefinition",
    "SyntheticNodeColumn",
    "SyntheticDataComputeNodeDefinition",
    "MaskType",
    "PreviewComputeNode",
    "PreviewComputeNodeDefinition",
    "FileContent",
    # Data nodes
    "RawDataNode",
    "RawDataNodeDefinition",
    "TableDataNode",
    "TableDataNodeDefinition",
    "Column",
    "PrimitiveType",
    "FormatType",
    "read_input_csv_file",
]



from __future__ import annotations
from dataclasses import dataclass
from datetime import datetime
import json
import chily
import hashlib
import hmac
from base64 import b64decode
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives import hashes
from typing import (
    Any,
    List,
    Tuple,
    TYPE_CHECKING,
    Iterator,
    Optional,
    Dict,
    Mapping,
    Text,
)
from time import sleep
from decentriq_platform.proto.gcg_pb2 import (
    GetResultsSizeRequest,
    RetrieveUsedAirlockQuotaRequest,
)
from .api import Endpoints
from .authentication import Auth, Sigma
from .proto import (
    DcrMetadata,
    CreateDcrPurpose,
    CreateDcrKind,
    DataRoom,
    ComputeNodeProtocol,
    DataNoncePubkey,
    Request,
    Response,
    CreateDataRoomRequest,
    CreateDataRoomResponse,
    ExecuteComputeRequest,
    ExecuteComputeResponse,
    GcgRequest,
    GcgResponse,
    GetResultsRequest,
    GetResultsResponseChunk,
    JobStatusRequest,
    JobStatusResponse,
    PublishDatasetToDataRoomRequest,
    PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest,
    RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest,
    RetrieveAuditLogResponse,
    RetrieveDataRoomRequest,
    RetrieveDataRoomResponse,
    RetrieveDataRoomStatusRequest,
    RetrieveConfigurationCommitResponse,
    RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse,
    UpdateDataRoomStatusRequest,
    ExecuteDevelopmentComputeRequest,
    CreateConfigurationCommitRequest,
    GenerateMergeApprovalSignatureRequest,
    MergeConfigurationCommitRequest,
    RetrieveCurrentDataRoomConfigurationRequest,
    UpdateDataRoomStatusResponse,
    DataRoomStatus,
    AttestationSpecification,
    Fatquote,
    CreateConfigurationCommitRequest,
    RetrieveConfigurationCommitApproversRequest,
    RetrieveConfigurationCommitRequest,
    ConfigurationCommit,
    MergeConfigurationCommitResponse,
    DataRoomConfiguration,
    EndorsementRequest,
    EndorsementResponse,
    Pki,
    PkiEndorsementRequest,
    PkiEndorsementResponse,
    DcrSecretEndorsementRequest,
    DcrSecretEndorsementResponse,
    TestDataset as TestDatasetProto,
)
from .proto.length_delimited import parse_length_delimited, serialize_length_delimited
from .storage import Key
from .types import DryRunOptions, JobId, DataRoomKind
from .verification import QuoteBody, Verification
from decentriq_dcr_compiler import compiler

if TYPE_CHECKING:
    from .client import Client


__all__ = [
    "Session",
    "LATEST_GCG_PROTOCOL_VERSION",
    "LATEST_WORKER_PROTOCOL_VERSION",
]


LATEST_GCG_PROTOCOL_VERSION = 6
LATEST_WORKER_PROTOCOL_VERSION = 1


@dataclass
class AirlockQuotaInfo:
    limit: int
    used: int


def _get_data_room_id(create_data_room_response: CreateDataRoomResponse) -> bytes:
    response_type = create_data_room_response.WhichOneof("create_data_room_response")
    if response_type is None:
        raise Exception("Empty CreateDataRoomResponse")
    elif response_type == "dataRoomId":
        return create_data_room_response.dataRoomId
    elif response_type == "dataRoomValidationError":
        raise Exception(
            "DataRoom creation failed",
            create_data_room_response.dataRoomValidationError,
        )
    else:
        raise Exception(
            "Unknown response type for CreateDataRoomResponse", response_type
        )


def datanoncepubkey_to_message(
    encrypted_data: bytes,
    nonce: bytes,
    pubkey: bytes,
) -> DataNoncePubkey:
    message = DataNoncePubkey()
    message.data = encrypted_data
    message.nonce = nonce
    message.pubkey = pubkey
    return message


def message_to_datanoncepubkey(message: bytes) -> Tuple[bytes, bytes, bytes]:
    parsed_msg = DataNoncePubkey()
    parse_length_delimited(message, parsed_msg)
    return (parsed_msg.data, parsed_msg.nonce, parsed_msg.pubkey)


class Session:
    """
    Class for managing the communication with an enclave.
    """

    client: Client
    session_id: str
    enclave_identifier: str
    auth: Auth
    email: str
    keypair: Any
    fatquote: Fatquote
    quote: QuoteBody
    driver_attestation_specification: AttestationSpecification
    client_protocols: List[int]

    def __init__(
        self,
        client: Client,
        session_id: str,
        driver_attestation_specification: AttestationSpecification,
        client_protocols: List[int],
        auth: Auth,
    ):
        """
        `Session` instances should not be instantiated directly but rather
         be created using a `Client` object using  `decentriq_platform.Client.create_session`.
        """
        data = client._graphql.post(
            """
            query GetFatquote($id: String!) {
                session(id: $id) {
                    fatquote
                }
            }
            """,
            {
                "id": session_id,
            },
        )

        fatquote = Fatquote()
        fatquote_bytes_encoded = b64decode(data["session"]["fatquote"])
        parse_length_delimited(fatquote_bytes_encoded, fatquote)
        verification = Verification(
            attestation_specification=driver_attestation_specification
        )
        if client.unsafe_disable_known_root_ca_check == True:
            verification.disable_known_root_ca_check()

        report_data = verification.verify(fatquote)
        self.client = client
        self.session_id = session_id
        self.auth = auth
        self.email = auth.user_id
        self.keypair = chily.Keypair.from_random()
        self.fatquote = fatquote
        self.report_data = report_data
        self.driver_attestation_specification = driver_attestation_specification
        self.driver_attestation_specification_hash = hashlib.sha256(
            serialize_length_delimited(driver_attestation_specification)
        ).hexdigest()
        self.client_protocols = client_protocols

    def _get_client_protocol(self, endpoint_protocols: List[int]) -> int:
        try:
            protocol = max(set(self.client_protocols) & set(endpoint_protocols))
            return protocol
        except ValueError:
            min_enclave_version = min(self.client_protocols)
            max_endpoint_version = min(endpoint_protocols)
            exception_message = "Endpoint is only available with protocol versions {} but the enclave only supports {}\n".format(
                endpoint_protocols, self.client_protocols
            )
            if min_enclave_version > max_endpoint_version:
                exception_message += "Try upgrading to a newer version of the SDK"
            else:
                exception_message += "Try using an older version of the SDK"
            raise Exception(exception_message)

    def _get_enclave_pubkey(self):
        pub_keyB = bytearray(self.report_data[:32])
        return chily.PublicKey.from_bytes(pub_keyB)

    def _get_message_pki(self, auth: Auth) -> Pki:
        shared_key = bytes(
            self.keypair.secret.diffie_hellman(self._get_enclave_pubkey()).bytes
        )
        hkdf = HKDF(
            algorithm=hashes.SHA512(), length=64, info=b"IdP KDF Context", salt=b""
        )
        mac_key = hkdf.derive(shared_key)
        mac_tag = hmac.digest(mac_key, auth._get_user_id().encode(), "sha512")
        public_keys = bytes(self.keypair.public_key.bytes) + bytes(
            self._get_enclave_pubkey().bytes
        )
        signature = auth._sign(public_keys)
        sigma_auth = Sigma(signature, mac_tag, auth)
        return Pki(
            certChainPem=sigma_auth.get_cert_chain(),
            signature=sigma_auth.get_signature(),
            idMac=sigma_auth.get_mac_tag(),
        )

    def _encrypt_and_encode_data(self, data: bytes) -> DataNoncePubkey:
        nonce = chily.Nonce.from_random()
        cipher = chily.Cipher(self.keypair.secret, self._get_enclave_pubkey())
        enc_data = cipher.encrypt("client sent session data", data, nonce)
        data_nonce_pubkey = datanoncepubkey_to_message(
            bytes(enc_data),
            bytes(nonce.bytes),
            bytes(self.keypair.public_key.bytes),
        )
        return data_nonce_pubkey

    def _decode_and_decrypt_data(self, data: bytes) -> bytes:
        dec_data, nonceB, _ = message_to_datanoncepubkey(data)
        cipher = chily.Cipher(self.keypair.secret, self._get_enclave_pubkey())
        return cipher.decrypt(
            "client received session data", dec_data, chily.Nonce.from_bytes(nonceB)
        )

    def _send_endorsement_request(
        self,
        request: EndorsementRequest,
        protocol: int,
    ) -> EndorsementResponse:
        responses = self.send_request(GcgRequest(endorsementRequest=request), protocol)
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]

        if not response.HasField("endorsementResponse"):
            raise Exception(
                "Expected endorsementResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.endorsementResponse

    def pki_endorsement(
        self,
        certificate_chain_pem: bytes,
    ) -> PkiEndorsementResponse:
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = PkiEndorsementRequest(certificateChainPem=certificate_chain_pem)
        response = self._send_endorsement_request(
            EndorsementRequest(pkiEndorsementRequest=request), protocol
        )

        if not response.HasField("pkiEndorsementResponse"):
            raise Exception(
                "Expected pkiEndorsementResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.pkiEndorsementResponse

    def dcr_secret_endorsement(
        self,
        dcr_secret: str,
    ) -> DcrSecretEndorsementResponse:
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = DcrSecretEndorsementRequest(
            dcrSecret=dcr_secret,
        )
        response = self._send_endorsement_request(
            EndorsementRequest(dcrSecretEndorsementRequest=request), protocol
        )

        if not response.HasField("dcrSecretEndorsementResponse"):
            raise Exception(
                "Expected dcrSecretEndorsementResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.dcrSecretEndorsementResponse

    def send_request(
        self,
        request: GcgRequest,
        protocol: int,
    ) -> List[GcgResponse]:
        """
        Low-level method for sending a raw `GcgRequest` to the enclave.
        Use this method if any of the convenience methods (such as `run_computation`) don't perform
        the exact task you want.
        """
        message_pki = self._get_message_pki(self.auth)
        request.userAuth.pki.CopyFrom(message_pki)
        request.userAuth.enclaveEndorsements.CopyFrom(self.auth.endorsements)
        gcg_protocol = serialize_length_delimited(ComputeNodeProtocol(version=protocol))
        serialized_request = serialize_length_delimited(
            Request(
                deltaRequest=self._encrypt_and_encode_data(
                    gcg_protocol + serialize_length_delimited(request)
                )
            )
        )
        url = Endpoints.SESSION_MESSAGES.replace(":sessionId", self.session_id)
        enclave_response: bytes = self.client._api.post(
            url,
            serialized_request,
            {"Content-type": "application/octet-stream", "Accept-Version": "2"},
        ).content

        responses: List[GcgResponse] = []
        offset = 0
        while offset < len(enclave_response):
            response_container = Response()
            offset += parse_length_delimited(
                enclave_response[offset:], response_container
            )
            if response_container.HasField("unsuccessfulResponse"):
                raise Exception(response_container.unsuccessfulResponse)
            else:
                response = GcgResponse()
                decrypted_response = self._decode_and_decrypt_data(
                    response_container.successfulResponse
                )
                response_protocol = ComputeNodeProtocol()
                response_offset = parse_length_delimited(
                    decrypted_response, response_protocol
                )
                if response_protocol.version != protocol:
                    raise Exception(
                        "Different response protocol version than requested"
                    )
                parse_length_delimited(decrypted_response[response_offset:], response)
                if response.HasField("failure"):
                    raise Exception(response.failure)
                responses.append(response)
        return responses

    def _publish_data_room(
        self,
        data_room_definition: DataRoom,
        purpose: CreateDcrPurpose.V = CreateDcrPurpose.STANDARD,
        kind: CreateDcrKind.V = CreateDcrKind.EXPERT,
        show_organization_logo: bool = False,
        require_password: bool = False,
        high_level_representation: Optional[bytes] = None,
    ) -> CreateDataRoomResponse:
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)

        metadata = DcrMetadata(
            showOrganizationLogo=show_organization_logo,
            requirePassword=require_password,
            purpose=purpose,
            kind=kind,
        )
        request = CreateDataRoomRequest(
            dataRoom=data_room_definition,
            highLevelRepresentation=high_level_representation,
            dataRoomMetadata=serialize_length_delimited(metadata),
        )
        responses = self.send_request(
            GcgRequest(createDataRoomRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]

        if response.HasField("createDataRoomResponse"):
            if response.createDataRoomResponse.HasField("dataRoomValidationError"):
                raise Exception(
                    "Error when validating data room: {} (compute node id '{}')".format(
                        response.createDataRoomResponse.dataRoomValidationError.message,
                        response.createDataRoomResponse.dataRoomValidationError.computeNodeId,
                    )
                )
        else:
            raise Exception(
                "Expected createDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.createDataRoomResponse

    def publish_data_room(
        self,
        data_room_definition: DataRoom,
        /,
        *,
        show_organization_logo: bool = False,
        require_password: bool = False,
        purpose: CreateDcrPurpose.V = CreateDcrPurpose.STANDARD,
        kind: CreateDcrKind.V = CreateDcrKind.EXPERT,
        high_level_representation: Optional[bytes] = None,
    ) -> str:
        """
        Create a data room with the provided protobuf configuration object
        and have the enclave apply the given list of modifications to the data
        room configuration.

        The id returned from this method will be used when interacting with the
        published data room (for example when running computations or publishing
        datasets).
        """
        response = self._publish_data_room(
            data_room_definition,
            purpose,
            kind,
            show_organization_logo,
            require_password,
            high_level_representation,
        )
        return _get_data_room_id(response).hex()

    def publish_data_room_configuration_commit(
        self, configuration_commit: ConfigurationCommit
    ) -> str:
        """
        Publish the given data room configuration commit.

        Configuration commits can be built using a `DataRoomCommitBuilder` object.

        The id returned from this method will be used when running development
        computations or when trying to merge this commit into the main
        data room configuration.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = CreateConfigurationCommitRequest(
            commit=configuration_commit,
        )
        responses = self.send_request(
            GcgRequest(createConfigurationCommitRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("createConfigurationCommitResponse"):
            raise Exception(
                "Expected createConfigurationCommitResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.createConfigurationCommitResponse.commitId.hex()

    def retrieve_configuration_commit(
        self,
        configuration_commit_id: str,
    ) -> RetrieveConfigurationCommitResponse:
        """
        Retrieve the content of given configuration commit id.

        **Returns**:
        A `ConfigurationCommit`.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveConfigurationCommitRequest(
            commitId=bytes.fromhex(configuration_commit_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveConfigurationCommitRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("retrieveConfigurationCommitResponse"):
            raise Exception(
                "Expected retrieveConfigurationCommitResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.retrieveConfigurationCommitResponse

    def retrieve_configuration_commit_approvers(
        self,
        configuration_commit_id: str,
    ) -> List[str]:
        """
        Retrieve the list of users who need to approve the merger of a given
        configuration commit.

        **Returns**:
        A list of ids belonging to the users that need to approve the
        configuration commit.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveConfigurationCommitApproversRequest(
            commitId=bytes.fromhex(configuration_commit_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveConfigurationCommitApproversRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("retrieveConfigurationCommitApproversResponse"):
            raise Exception(
                "Expected retrieveConfigurationCommitApproversResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return list(response.retrieveConfigurationCommitApproversResponse.approvers)

    def generate_merge_approval_signature(self, configuration_commit_id: str) -> bytes:
        """
        Generate an approval signature required for merging a configuration
        commit.

        To merge a specific configuration commit, each user referenced in the list
        of ids returned by `retrieveConfigurationCommitApprovers` needs to
        generate an approval signature using this method.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = GenerateMergeApprovalSignatureRequest(
            commitId=bytes.fromhex(configuration_commit_id),
        )
        responses = self.send_request(
            GcgRequest(generateMergeApprovalSignatureRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("generateMergeApprovalSignatureResponse"):
            raise Exception(
                "Expected generateMergeApprovalSignatureResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.generateMergeApprovalSignatureResponse.signature

    def merge_configuration_commit(
        self,
        configuration_commit_id: str,
        approval_signatures: Dict[str, bytes],
    ) -> MergeConfigurationCommitResponse:
        """
        Request the enclave to merge the given configuration commit into the
        main data room configuration.

        **Parameters**:
        - `configuration_commit_id`: The id of the commit to be merged.
        - `approval_signatures`: A dictionary containing the approval signature for
            each of the required approvers, e.g. `{ "some@email.com": signature }`.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = MergeConfigurationCommitRequest(
            commitId=bytes.fromhex(configuration_commit_id),
            approvalSignatures=approval_signatures,
        )
        responses = self.send_request(
            GcgRequest(mergeConfigurationCommitRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("mergeConfigurationCommitResponse"):
            raise Exception(
                "Expected mergeConfigurationCommitResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.mergeConfigurationCommitResponse

    def retrieve_current_data_room_configuration(
        self, data_room_id: str
    ) -> Tuple[DataRoomConfiguration, str]:
        """
        Retrieve the current data room confguration, as well as the current "history pin".

        A history pin is the hash of all the ids of configuration commits that
        make up the structure of a data room. This pin therefore uniquely identifies
        a data room's structure at a certain point in time.
        A data room configuration, as well as its associated history pin, can be used
        to extend an existing data room (for example by adding new compute nodes).
        Extending an existing data room is done using the `DataRoomCommitBuilder` class.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveCurrentDataRoomConfigurationRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveCurrentDataRoomConfigurationRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        gcg_response = responses[0]
        if not gcg_response.HasField("retrieveCurrentDataRoomConfigurationResponse"):
            raise Exception(
                "Expected retrieveCurrentDataRoomConfigurationResponse, got "
                + str(gcg_response.WhichOneof("gcg_response"))
            )

        response = gcg_response.retrieveCurrentDataRoomConfigurationResponse
        return (response.configuration, response.pin.hex())

    def stop_data_room(self, data_room_id: str):
        """
        Stop the data room with the given id, making it impossible to run new
        computations.
        """
        self._update_data_room_status(data_room_id, DataRoomStatus.Value("Stopped"))

    def _update_data_room_status(
        self, data_room_id: str, status  # type: DataRoomStatus.V
    ) -> UpdateDataRoomStatusResponse:
        """
        Update the status of the data room.

        For the special case of stopping a data room, the method
        `stop_data_room` can be used.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = UpdateDataRoomStatusRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            status=status,
        )
        responses = self.send_request(
            GcgRequest(updateDataRoomStatusRequest=request), protocol
        )

        if len(responses) != 1:
            raise Exception("Malformed response")

        response = responses[0]
        if not response.HasField("updateDataRoomStatusResponse"):
            raise Exception(
                "Expected updateDataRoomStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.updateDataRoomStatusResponse

    def retrieve_data_room_status(self, data_room_id: str) -> str:
        """
        Returns the status of the data room. Valid values are `"Active"` or `"Stopped"`.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveDataRoomStatusRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveDataRoomStatusRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveDataRoomStatusResponse"):
            raise Exception(
                "Expected retrieveDataRoomStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return DataRoomStatus.Name(response.retrieveDataRoomStatusResponse.status)

    def retrieve_data_room(self, data_room_id: str) -> RetrieveDataRoomResponse:
        """
        Returns the underlying protobuf object for the data room.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveDataRoomRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveDataRoomRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveDataRoomResponse"):
            raise Exception(
                "Expected retrieveDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrieveDataRoomResponse

    def retrieve_audit_log(self, data_room_id: str) -> RetrieveAuditLogResponse:
        """
        Returns the audit log for the data room.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveAuditLogRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveAuditLogRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveAuditLogResponse"):
            raise Exception(
                "Expected retrieveAuditLogResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrieveAuditLogResponse

    def retrieve_data_room_json(self, data_room_id: str) -> str:
        """
        Get the JSON configuration file for the data room with the given ID.
        Returns a JSON string representing the configuration.
        """
        dcr_kind = self.client._get_data_room_kind(data_room_id)
        if dcr_kind == DataRoomKind.DATA_SCIENCE:
            dcr = self.retrieve_data_room(data_room_id)
            commits = [serialize_length_delimited(c) for c in dcr.commits]
            verified_dcr = compiler.verify_data_room(
                serialize_length_delimited(dcr.dataRoom),
                commits,
                dcr.highLevelRepresentation,
            )
            latest_dcr = compiler.upgrade_data_science_data_room_to_latest(verified_dcr)
            output = {
                # Format timestamp as ISO standard UTC time.
                "createdAt": str(datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f"))[:-3]
                + "Z",
                "dataScienceDataRoom": json.loads(latest_dcr.json()),
            }
            return json.dumps(output, indent=2)
        else:
            raise Exception(f"Cannot retrieve JSON for data room kind {dcr_kind}")

    def publish_dataset(
        self,
        data_room_id: str,
        manifest_hash: str,
        leaf_id: str,
        key: Key,
        *,
        force: bool = False,
    ) -> PublishDatasetToDataRoomResponse:
        """
        Publishes a file and its encryption key to a data room.
        Neither the file or the encryption key will ever be stored in
        unencrypted form.

        This method will check whether the to-be-published file exists.
        If this is not the case, an exception will be raised.
        This behavior can be disabled by setting the `force` flag.

        In case the original client was created with platform integration
        enabled, the method will further check whether there already
        is a dataset published for the given data room.
        In this case, an exception will be thrown and the dataset
        will need to be unpublished first.

        A special note for when the referenced data room was created using the Decentriq UI:
        In this case, the `leaf_id` argument will have the format `{NODE_ID}_leaf`,
        where `{NODE_ID}` corresponds to the value that you see when hovering your mouse pointer over
        the name of the data node.
        """

        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        dataset = self.client.get_dataset(manifest_hash)
        if not dataset and not force:
            raise Exception("The dataset you are trying to publish does not exist")

        scope_id = self.client._ensure_dcr_data_scope(
            data_room_id,
        )
        scope_id_bytes = bytes.fromhex(scope_id)
        request = PublishDatasetToDataRoomRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            datasetHash=bytes.fromhex(manifest_hash),
            leafId=leaf_id,
            encryptionKey=key.material,
            scope=scope_id_bytes,
        )
        responses = self.send_request(
            GcgRequest(publishDatasetToDataRoomRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("publishDatasetToDataRoomResponse"):
            raise Exception(
                "Expected publishDatasetToDataRoomResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.publishDatasetToDataRoomResponse

    def remove_published_dataset(
        self,
        data_room_id: str,
        leaf_id: str,
    ) -> RemovePublishedDatasetResponse:
        """
        Removes a published dataset from the data room.

        **Parameters**:
        - `data_room_id`: The ID of the data room that contains the given data set.
        - `leaf_id`: The ID of the data node from which the dataset should be removed.
            In case the referenced data room was created using the Decentriq UI,
            the `leaf_id` argument will have the special format `@table/UUID/dataset`
            (where `UUID` corresponds to the value that you see when hovering your mouse pointer over
            the name of the data node).
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RemovePublishedDatasetRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            leafId=leaf_id,
        )
        responses = self.send_request(
            GcgRequest(removePublishedDatasetRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("removePublishedDatasetResponse"):
            raise Exception(
                "Expected removePublishedDatasetResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )

        return response.removePublishedDatasetResponse

    def retrieve_published_datasets(
        self,
        data_room_id: str,
    ) -> RetrievePublishedDatasetsResponse:
        """
        Returns the datasets published to the given data room.
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrievePublishedDatasetsRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrievePublishedDatasetsRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrievePublishedDatasetsResponse"):
            raise Exception(
                "Expected retrievePublishedDatasetResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.retrievePublishedDatasetsResponse

    def _submit_dev_compute(
        self,
        data_room_id: str,
        configuration_commit_id: str,
        compute_node_ids: List[str],
        /,
        *,
        dry_run: Optional[DryRunOptions] = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ) -> ExecuteComputeResponse:
        """
        Submits a computation request which will generate an execution plan to
        perform the computation of the goal nodes
        """
        endpoint_protocols = [5, 6]
        if (
            not dry_run
            or not "test_datasets" in dry_run
            or not dry_run["test_datasets"]
        ):
            endpoint_protocols.append(3)
            endpoint_protocols.append(4)

        protocol = self._get_client_protocol(endpoint_protocols)
        scope_id = self.client._ensure_dcr_data_scope(
            data_room_id,
        )
        scope_id_bytes = bytes.fromhex(scope_id)

        is_dry_run = dry_run is not None
        test_datasets_map = (
            dry_run["test_datasets"] if dry_run and "test_datasets" in dry_run else {}
        )
        test_datasets = {
            k: TestDatasetProto(
                manifestHash=bytes.fromhex(v["manifest_hash"]),
                encryptionKey=v["key"].material,
            )
            for k, v in test_datasets_map.items()
        }

        request = ExecuteDevelopmentComputeRequest(
            configurationCommitId=bytes.fromhex(configuration_commit_id),
            computeNodeIds=compute_node_ids,
            isDryRun=is_dry_run,
            scope=scope_id_bytes,
            testDatasets=test_datasets,
            parameters=parameters,
        )
        responses = self.send_request(
            GcgRequest(executeDevelopmentComputeRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("executeComputeResponse"):
            raise Exception(
                "Expected executeComputeResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.executeComputeResponse

    def _submit_compute(
        self,
        data_room_id: str,
        compute_node_ids: List[str],
        /,
        *,
        dry_run: Optional[DryRunOptions] = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ) -> ExecuteComputeResponse:
        """
        Submits a computation request which will generate an execution plan to
        perform the computation of the goal nodes
        """
        endpoint_protocols = [5, 6]
        if (
            not dry_run
            or not "test_datasets" in dry_run
            or not dry_run["test_datasets"]
        ):
            endpoint_protocols.append(3)
            endpoint_protocols.append(4)

        protocol = self._get_client_protocol(endpoint_protocols)
        scope_id = self.client._ensure_dcr_data_scope(
            data_room_id,
        )
        scope_id_bytes = bytes.fromhex(scope_id)

        is_dry_run = dry_run is not None
        test_datasets_map = (
            dry_run["test_datasets"] if dry_run and "test_datasets" in dry_run else {}
        )
        test_datasets = {
            k: TestDatasetProto(
                manifestHash=bytes.fromhex(v["manifest_hash"]),
                encryptionKey=v["key"].material,
            )
            for k, v in test_datasets_map.items()
        }
        request = ExecuteComputeRequest(
            dataRoomId=bytes.fromhex(data_room_id),
            computeNodeIds=compute_node_ids,
            isDryRun=is_dry_run,
            scope=scope_id_bytes,
            testDatasets=test_datasets,
            parameters=parameters,
        )
        responses = self.send_request(
            GcgRequest(executeComputeRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("executeComputeResponse"):
            raise Exception(
                "Expected executeComputeResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.executeComputeResponse

    def get_computation_status(self, job_id: str) -> JobStatusResponse:
        """
        Returns the status of the provided `job_id` which will include the names
        of the nodes that completed their execution
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = JobStatusRequest(
            jobId=bytes.fromhex(job_id),
        )
        responses = self.send_request(GcgRequest(jobStatusRequest=request), protocol)
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("jobStatusResponse"):
            raise Exception(
                "Expected jobStatusResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.jobStatusResponse

    def _stream_job_results(
        self,
        job_id: bytes,
        compute_node_id: str,
    ) -> Iterator[GetResultsResponseChunk]:
        """
        Streams the results of the provided `job_id`
        """
        endpoint_protocols = [3, 4, 5, 6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = GetResultsRequest(
            jobId=job_id,
            computeNodeId=compute_node_id,
        )
        responses = self.send_request(GcgRequest(getResultsRequest=request), protocol)
        for response in responses:
            if response.HasField("getResultsResponseChunk"):
                yield response.getResultsResponseChunk
            elif response.HasField("getResultsResponseFooter"):
                return
            else:
                raise Exception(
                    "Expected getResultsResponseChunk or getResultsResponseFooter, got "
                    + str(response.WhichOneof("gcg_response"))
                )
        raise Exception("Enclave connection aborted while streaming results")

    def _get_job_results(
        self,
        job_id: bytes,
        compute_node_id: str,
    ) -> bytes:
        """
        Returns the results of the provided `job_id`
        """
        return b"".join(
            list(
                map(
                    lambda chunk: chunk.data,
                    self._stream_job_results(job_id, compute_node_id),
                )
            )
        )

    def run_computation(
        self,
        data_room_id: str,
        compute_node_id: str,
        /,
        *,
        dry_run: Optional[DryRunOptions] = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ) -> JobId:
        """
        Run a specific computation within the data room with the given id.

        The result will be an identifier object of the job executing the computation.
        This object is required for checking a job's status and retrieving its results.
        """
        response = self._submit_compute(
            data_room_id, [compute_node_id], dry_run=dry_run, parameters=parameters
        )
        return JobId(response.jobId.hex(), compute_node_id)

    def wait_until_computation_has_finished(
        self, job_id: JobId, /, *, interval: int = 5, timeout: int = None
    ):
        """
        Wait for the given job to complete.

        The method will check for the job's completeness every `interval` seconds and up to
        an optional `timeout` seconds after which the method will raise an exception.
        """
        elapsed = 0
        while True:
            if timeout is not None and elapsed > timeout:
                raise Exception(
                    f"Timeout when trying to get result for job {job_id.id} of"
                    f" {job_id.compute_node_id} (waited {timeout} seconds)"
                )
            elif (
                job_id.compute_node_id
                in self.get_computation_status(job_id.id).completeComputeNodeIds
            ):
                break
            else:
                sleep(interval)
                elapsed += interval

    def wait_until_computation_has_finished_for_all_compute_nodes(
        self,
        job_id: str,
        compute_node_ids: List[str],
        /,
        *,
        interval: int = 5,
        timeout: int = None,
    ):
        """
        Wait for the given job to complete for all of the given compute nodes.

        The method will check for the job's completeness every `interval` seconds and up to
        an optional `timeout` seconds after which the method will raise an exception.
        """
        elapsed = 0
        while True:
            if timeout is not None and elapsed > timeout:
                raise Exception(
                    f"Timeout when trying to get result for job {job_id} (waited {timeout} seconds)"
                )
            elif set(compute_node_ids).issubset(
                self.get_computation_status(job_id).completeComputeNodeIds
            ):
                break
            else:
                sleep(interval)
                elapsed += interval

    def run_dev_computation(
        self,
        data_room_id: str,
        configuration_commit_id: str,
        compute_node_id: str,
        /,
        *,
        dry_run: Optional[DryRunOptions] = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ) -> JobId:
        """
        Run a specific computation within the context of the data room configuration
        defined by the given commit id.
        Such "development" computations can also be run for configuration commits
        that have not yet been merged.

        The result will be an identifier object of the job executing the computation.
        This object is required for checking a job's status and retrieving its results.
        """
        response = self._submit_dev_compute(
            data_room_id,
            configuration_commit_id,
            [compute_node_id],
            dry_run=dry_run,
            parameters=parameters,
        )
        return JobId(response.jobId.hex(), compute_node_id)

    def get_computation_result(
        self,
        job_id: JobId,
        /,
        *,
        interval: int = 5,
        timeout: int = None,
    ) -> bytes:
        """
        Wait for the given job to complete and retrieve its results as a raw byte string.

        The method will check for the job's completeness every `interval` seconds and up to
        an optional `timeout` seconds after which the method will raise an exception.
        If the job completes and the results can be retrieved successfully, a raw byte string
        will be returned. The bytes string can be transformed into a more useful object using
        a variety of helper methods. These helper methods are specific for the type of computation
        you ran and can be found in the corresponding packages.
        """
        job_id_bytes = bytes.fromhex(job_id.id)
        self.wait_until_computation_has_finished(
            job_id, interval=interval, timeout=timeout
        )
        results = self._get_job_results(job_id_bytes, job_id.compute_node_id)
        return results

    def get_computation_result_size(
        self,
        job_id: JobId,
        /,
        *,
        interval: int = 5,
        timeout: int = None,
    ) -> int:
        """
        Wait for the given job to complete and retrieve its results size.

        The method will check for the job's completeness every `interval` seconds and up to
        an optional `timeout` seconds after which the method will raise an exception.
        If the job completes and the results can be retrieved successfully, an int containing
        the raw result size is returned.
        """
        endpoint_protocols = [6]
        protocol = self._get_client_protocol(endpoint_protocols)
        job_id_bytes = bytes.fromhex(job_id.id)
        self.wait_until_computation_has_finished(
            job_id, interval=interval, timeout=timeout
        )
        request = GetResultsSizeRequest(
            jobId=job_id_bytes,
            computeNodeId=job_id.compute_node_id,
        )
        responses = self.send_request(
            GcgRequest(getResultsSizeRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("getResultsSizeResponse"):
            raise Exception(
                "Expected getResultsSizeResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return response.getResultsSizeResponse.sizeBytes

    def run_computation_and_get_results(
        self,
        data_room_id: str,
        compute_node_id: str,
        /,
        *,
        interval: int = 5,
        timeout: int = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ) -> Optional[bytes]:
        """
        Run a specific computation and return its results.

        This method is simply a wrapper for running `run_computation` and
        `get_computation_result` directly after each other
        """
        job_id = self.run_computation(
            data_room_id, compute_node_id, parameters=parameters
        )
        return self.get_computation_result(job_id, interval=interval, timeout=timeout)

    def retrieve_used_airlock_quotas(
        self,
        data_room_id: str,
    ) -> Dict[str, AirlockQuotaInfo]:
        """
        Retrieves the limit and used airlock quota for the current user.
        """
        endpoint_protocols = [6]
        protocol = self._get_client_protocol(endpoint_protocols)
        request = RetrieveUsedAirlockQuotaRequest(
            dataRoomId=bytes.fromhex(data_room_id),
        )
        responses = self.send_request(
            GcgRequest(retrieveUsedAirlockQuotaRequest=request), protocol
        )
        if len(responses) != 1:
            raise Exception("Malformed response")
        response = responses[0]
        if not response.HasField("retrieveUsedAirlockQuotaResponse"):
            raise Exception(
                "Expected retrieveUsedAirlockQuotaResponse, got "
                + str(response.WhichOneof("gcg_response"))
            )
        return {
            airlock_quota.airlockNodeId: AirlockQuotaInfo(
                airlock_quota.quotaBytes, airlock_quota.usedQuotaBytes
            )
            for airlock_quota in response.retrieveUsedAirlockQuotaResponse.airlockQuotas
        }
